## -*- coding: utf-8 -*-
#
# «roles» - MCC System Role selector plugin
#
# Copyright (C) 2009, Mario Limonciello, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin

from mythbuntu_common.dictionaries import *

class SystemRolesPlugin(MCCPlugin):
    """A tool for adjusting the role of a system"""

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "System Roles"
        information["icon"] = "gnome-monitor"
        information["ui"] = "tab_system_roles"
        MCCPlugin.__init__(self,information)

    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""
        #We can't really represent no backend or no frontend well yet
        self.no_back=True
        self.no_front=True

        #Dictionaries
        self.dictionary_state={}
        for list in get_role_dictionary(self), \
                    get_diskless_dictionary(self), \
                    get_desktop_dictionary(self):
            for item in list:
                self.dictionary_state[list[item]]=self.query_installed(item)
                if "backend" in item and self.dictionary_state[list[item]]:
                    self.no_back=False
                elif "frontend" in item and self.dictionary_state[list[item]]:
                    self.no_front=False

        #corner case
        if self.dictionary_state[self.primary_backend_radio]:
            self.dictionary_state[self.secondary_backend_radio]=False

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""
        #Load the detected dictionary
        for item in self.dictionary_state:
            if self.dictionary_state[item]:
                item.set_active(True)

        #In case we don't have a front or back role
        self.no_backend_radio.set_active(self.no_back)
        self.no_frontend_radio.set_active(self.no_front)

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        #Prepare for state capturing
        MCCPlugin.clearParentState(self)

        #Installable items
        for list in get_diskless_dictionary(self), \
                    get_desktop_dictionary(self):
            for item in list:
                if list[item].get_active() != self.dictionary_state[list[item]]:
                    if list[item].get_active():
                        self._markInstall(item)
                    else:
                        self._markRemove(item)

        #backend unfortunately totally a corner case
        if self.primary_backend_radio.get_active() != self.dictionary_state[self.primary_backend_radio]:
            if self.primary_backend_radio.get_active():
                self._markInstall('mythtv-backend-master')
            else:
                self._markRemove('mythtv-backend-master')
                if not self.secondary_backend_radio.get_active():
                    self._markRemove('mythtv-backend')
        elif self.secondary_backend_radio.get_active() != self.dictionary_state[self.secondary_backend_radio]:
            if self.secondary_backend_radio.get_active():
                self._markInstall('mythtv-backend')
            else:
                self._markRemove('mythtv-backend')

        if self.frontend_radio.get_active() != self.dictionary_state[self.frontend_radio]:
            if self.frontend_radio.get_active():
                self._markInstall('mythtv-frontend')
            else:
                self._markRemove('mythtv-frontend')





## -*- coding: utf-8 -*-
#
# «proprietary_codecs» - MCC Proprietary Codecs enablement plugin
#
# Copyright (C) 2009, Mario Limonciello, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin
import subprocess
import re
import os
import logging
from mythbuntu_common.dictionaries import *

class ProprietaryCodecsPlugin(MCCPlugin):
    """Proprietary Codec Enablement"""

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "Proprietary Codecs"
        information["icon"] = "gtk-dialog-warning"
        information["ui"] = "tab_proprietary_codecs"
        MCCPlugin.__init__(self,information)

    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""
        #Dictionaries
        self.dictionary_state={}
        list=get_nonfree_dictionary(self)
        for item in list:
            self.dictionary_state[list[item]]=self.query_installed(item)

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""
        #Load the detected dictionary
        for item in self.dictionary_state:
            item.set_active(self.dictionary_state[item])

        self.proprietary_codecs.set_sensitive(True)

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        #Prepare for state capturing
        MCCPlugin.clearParentState(self)

        #Installable items
        list=get_nonfree_dictionary(self)
        for item in list:
            if list[item].get_active() != self.dictionary_state[list[item]]:
                if list[item].get_active():
                    self._markReconfigureRoot("DVD Support",self.enable_libdvdcss2.get_active())
                else:
                    self._markRemove(item)


    def search(self,path, filename):
        for root, dirs, files in os.walk(path):
            if filename in files:
                rootpath = root
        return rootpath

    def root_scripted_changes(self,reconfigure):
        """System-wide changes that need root access to be applied.
           This function is ran by the dbus backend"""
        os.environ['PATH']='/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin'
        for item in reconfigure:
            if item == "DVD Support":
                csspath = self.search("/usr/share/doc/", "install-css.sh")
                run = subprocess.Popen([csspath+'/install-css.sh'])
                run.communicate()



## -*- coding: utf-8 -*-
#
# «MythExport» - A MythExport Plugin for MCC
#
# Copyright (C) 2009, John Baab, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin
from gi.repository import Gtk
import os

class MythExportPlugin(MCCPlugin):
    """A plugin to enable or disable mythexport"""
    #
    #Load GUI & Calculate Changes
    #
    CONFIGFILE = "/etc/default/mythexport-install"

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "MythExport"
        information["icon"] = "gtk-leave-fullscreen"
        information["ui"] = "tab_mythexport"
        MCCPlugin.__init__(self,information)
        import configparser
        self.config = configparser.ConfigParser()

    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""
        import os
        self.changes = {}
        self.changes['mythexport'] = self.query_installed('mythexport')

        if os.path.exists(self.CONFIGFILE):
            self.config.read(self.CONFIGFILE)
            self.changes['dir'] = self.config.get("cfg", "dir")
        else:
            self.changes['dir'] = "/var/lib/mythtv/"

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""
        self.mythexport_checkbox.set_active(self.changes['mythexport'])
        self.export_dir.set_current_folder(self.changes['dir'])
        if self.changes['mythexport']:
            self.configuration_linkbutton.show()
            self.export_dir.set_sensitive(True);
        else:
            self.configuration_linkbutton.hide()
            self.export_dir.set_sensitive(False);

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        MCCPlugin.clearParentState(self)
        if self.mythexport_checkbox.get_active() != self.changes['mythexport']:
            if self.mythexport_checkbox.get_active():
                self._markInstall('mythexport') 
                self._markReconfigureRoot('dir',self.export_dir.get_filename())            
            else:
                self._markRemove('mythexport')
        elif self.mythexport_checkbox.get_active() == True and self.export_dir.get_filename() != self.changes['dir']:
            self._markReconfigureRoot('dir',self.export_dir.get_filename())

    #
    # Process selected activities
    #

    def root_scripted_changes(self,reconfigure):
        """System-wide changes that need root access to be applied.
           This function is ran by the dbus backend"""
        if os.path.exists(self.CONFIGFILE):
          self.config.read(self.CONFIGFILE)
        else:
          self.config.add_section("cfg")
        for item in reconfigure:
            self.config.set("cfg", item, reconfigure[item]) 

        with open(self.CONFIGFILE, 'wb') as configfile:
          self.config.write(configfile)
        os.system("DEBIAN_FRONTEND=noninteractive dpkg-reconfigure mythexport")





## -*- coding: utf-8 -*-
#
# «startup_behavior» - MCC Automatic Login enablement plugin
#
# Copyright (C) 2009, Mario Limonciello, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin
import os
import string
import re

class LoginPlugin(MCCPlugin):
    """A plugin for setting up automatic login"""

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "Startup Behavior"
        information["icon"] = "emblem-desktop"
        information["ui"] = "tab_startup_behavior"
        self.user_count=0
        MCCPlugin.__init__(self,information)

    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""
        self.autostart_state=os.path.exists(os.environ['HOME'] + '/.config/autostart/mythtv.desktop')

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""
        self.enableautostartup.set_active(self.autostart_state)

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        #Prepare for state capturing
        MCCPlugin.clearParentState(self)

        if self.autostart_state != self.enableautostartup.get_active():
            self._markReconfigureUser("autostartup",self.enableautostartup.get_active())

    def user_scripted_changes(self,reconfigure):
        """Local changes that can be performed by the user account.
           This function will be ran by the frontend"""
        for item in reconfigure:
            if item == 'autostartup':
                home = os.environ['HOME']
                if reconfigure[item]:
                    if not os.path.exists(home + '/.config/autostart'):
                        os.makedirs(home + '/.config/autostart')
                    if not os.path.exists(home + '/.config/autostart/mythtv.desktop'):
                        try:
                            os.symlink('/usr/share/applications/mythtv.desktop',home + '/.config/autostart/mythtv.desktop')
                        except OSError:
                            os.unlink(home + '/.config/autostart/mythtv.desktop')
                            os.symlink('/usr/share/applications/mythtv.desktop',home + '/.config/autostart/mythtv.desktop')
                elif os.path.exists(home + '/.config/autostart/mythtv.desktop'):
                    os.remove(home + '/.config/autostart/mythtv.desktop')



## -*- coding: utf-8 -*-
#
# «plugins» - MCC & MythTV Plugin enablement plugin
#
# Copyright (C) 2009, Mario Limonciello, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin
from gi.repository import Gtk
import os
import string
import logging
import configparser

from mythbuntu_common.dictionaries import *

class MythPluginsPlugin(MCCPlugin):
    """A tool for enabling different MythTV plugins"""

    CONFIGFILE = "/etc/default/mythweb"

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "Plugins"
        information["icon"] = "gtk-add"
        information["ui"] = "tab_plugins"
        self.config = configparser.ConfigParser()
        MCCPlugin.__init__(self,information)


    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""
        #Dictionaries
        self.dictionary_state={}
        for list in get_frontend_plugin_dictionary(self), \
                    get_backend_plugin_dictionary(self):
            for item in list:
                self.dictionary_state[list[item]]=self.query_installed(item)

        #Mythweb auth
        self.mythweb_auth={}
        found_cfg=False
        if os.path.exists(self.CONFIGFILE):
            self.config.read(self.CONFIGFILE)
            try:
                self.mythweb_auth['enable'] = self.config.getboolean("cfg", "enable")
                self.mythweb_auth['user'] = self.config.getboolean("cfg", "username")
                self.mythweb_auth['pass'] = self.config.getboolean("cfg", "password")
                found_cfg=True
            except Exception:
                pass
        if not found_cfg:
            self.mythweb_auth['enable'] = os.path.exists('/etc/mythtv/mythweb-digest')
            self.mythweb_auth['user'] = ""
            self.mythweb_auth['pass'] = ""

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""

        #Load the detected dictionary
        for item in self.dictionary_state:
            item.set_active(self.dictionary_state[item])

        #Mythweb auth
        self.password_table.hide()
        self.mythweb_username.set_text("")
        self.mythweb_password.set_text("")
        model = self.mythweb_password_combobox.get_model()
        if len(model) > 2:
            iter = model.get_iter(Gtk.TreePath([2,0]))
            model.remove(iter)
        if self.mythweb_auth['enable']:
            self.mythweb_password_combobox.set_active_iter(model.get_iter(Gtk.TreePath([0,1])))
            self.mythweb_password_combobox.append_text("Reconfigure")
        else:
            self.mythweb_password_combobox.set_active_iter(model.get_iter(Gtk.TreePath([0,0])))

        self.toggle_plugins(self.mythweb_checkbox)

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        #Prepare for state capturing
        MCCPlugin.clearParentState(self)

        #Installable items
        for list in get_frontend_plugin_dictionary(self), \
                    get_backend_plugin_dictionary(self):
            for item in list:
                if list[item].get_active() != self.dictionary_state[list[item]]:
                    if list[item].get_active():
                        self._markInstall(item)
                    else:
                        self._markRemove(item)

        #Mythweb auth
        if self.mythweb_password_combobox.get_active() != self.mythweb_auth['enable']:
            self._markReconfigureRoot("mythweb_auth",self.mythweb_password_combobox.get_active() > 0)
        if self.mythweb_password_combobox.get_active():
            if self.mythweb_username.get_text() != self.mythweb_auth['user']:
                self._markReconfigureRoot("mythweb_user",self.mythweb_username.get_text())
            if self.mythweb_password.get_text() != self.mythweb_auth['pass']:
                self._markReconfigureRoot("mythweb_password",self.mythweb_password.get_text())

    def toggle_plugins(self,widget):
        if widget is not None:
            if widget.get_name() == 'selectall_plugins_checkbox':
                toggle=widget.get_active()
                for list in get_frontend_plugin_dictionary(self), get_backend_plugin_dictionary(self):
                    for item in list:
                        if list[item].flags() & Gtk.SENSITIVE:
                            list[item].set_active(toggle)

            elif widget.get_name() == "mythweb_checkbox":
                self.mythweb_password_combobox.set_sensitive(widget.get_active())
                if not widget.get_active():
                    self.mythweb_password_combobox.set_active(0)

            elif widget.get_name() == "mythweb_username" or \
                 widget.get_name() == "mythweb_password":
                username = self.mythweb_username.get_text().split(' ')[0]
                password = self.mythweb_password.get_text().split(' ')[0]
                if self.mythweb_password_combobox.get_active() != 2 or \
                   (len(username) > 0 and len(password) > 0):
                    self._incomplete=False
                else:
                    self._incomplete=True

            elif widget.get_name() == "mythweb_password_combobox":
                iteration=1
                if self.mythweb_auth['enable']:
                    iteration = 2
                if widget.get_active() == iteration:
                    self.password_table.show()
                    self._incomplete=True
                else:
                    self.mythweb_username.set_text("")
                    self.mythweb_password.set_text("")
                    self.password_table.hide()
                    self._incomplete=False

    def root_scripted_changes(self,reconfigure):
        """System-wide changes that need root access to be applied.
           This function is ran by the dbus backend"""

        found_cfg = False
        if os.path.exists(self.CONFIGFILE):
            try:
                self.config.read(self.CONFIGFILE)
                found_cfg = True
            except Exception:
                pass
        if not found_cfg:
            self.config.add_section("cfg")
            self.config.set("cfg", "enable", "false")
            self.config.set("cfg", "only", "false")
            self.config.set("cfg", "username", "")
            self.config.set("cfg", "password", "")

        for item in reconfigure:
            if item == "mythweb_auth":
                if not reconfigure[item]:
                    if os.path.exists('/etc/mythtv/mythweb-digest'):
                        os.remove('/etc/mythtv/mythweb-digest')
                    self.config.set("cfg", "enable", "false")
                else:
                    self.config.set("cfg", "enable", "true")
            elif item == "mythweb_user":
                self.config.set("cfg", "username", reconfigure[item])
            elif item == "mythweb_password":
                self.config.set("cfg", "password", reconfigure[item])

        with open(self.CONFIGFILE, 'wb') as configfile:
            self.config.write(configfile)
        os.system("dpkg-reconfigure -fnoninteractive mythweb")




# -*- coding: utf-8 -*-
#
# «lirc» - libmythbuntu class for lirc modifications
#
# This script:
# Copyright (C) 2010, John Baab, for Mythbuntu
# Copyright (C) 2007-2008, Mario Limonciello, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import re
import subprocess
import os
import shutil

#default remote hwdb
REMOTE_HWDB="/usr/share/lirc/lirc.hwdb"
EXTRA_REMOTE_HWDB="/usr/share/lirc/extras/more_remotes.hwdb"
TRANSMITTER_HWDB="/usr/share/lirc/extras/transmitter.hwdb"

#path to hardware.conf
HARDWARE_CONF="/etc/lirc/hardware.conf"

#Path to remotes
REMOTE_PATH="/usr/share/lirc/remotes/"

#path to main lircd.conf
LIRCD_CONF="/etc/lirc/lircd.conf"

class LircHandler:

    def __init__(self,hw_conf=HARDWARE_CONF):
        #initialize all lists
        self.remote_list=[]
        self.remote="None"
        self.remote_modules=""
        self.remote_device=""
        self.remote_driver=""
        self.remote_lircd_conf=""
        self.remote_count=0
        self.transmitter_list=[]
        self.transmitter="None"
        self.transmitter_modules=""
        self.transmitter_device=""
        self.transmitter_driver=""
        self.transmitter_lircd_conf=""
        self.transmitter_count=0
        self.populate_devices("remote",REMOTE_HWDB)
        self.populate_devices("remote",EXTRA_REMOTE_HWDB)
        self.populate_devices("transmitter",TRANSMITTER_HWDB)
        self.sort_devices()
        #load our current data
        self.read_hardware_conf(hw_conf)

    def sort_devices(self):
        """Sorts the device list for a type"""
        self.remote_list.sort(reverse=True)
        self.transmitter_list.sort(reverse=True)
        self.remote_list.insert(0,"None")
        self.transmitter_list.insert(0,"None")

    def populate_devices(self,type,file):
        """Populates all possible remotes from a hwdb.  Accepts either 'remote' or 'transmitter' for possible types"""
        try:
            hwdb = open(file).readlines()
            hwdb.sort()
            #Filter out uncessary lines
            filter = "^\#|^\["
            #Filter out the /dev/input/eventX remote
            filter += "|http"
            #Filter blaster if we are in remote
            if type == 'remote':
                filter += "|Blaster"
            pattern = re.compile(filter)
            for line in hwdb:
                if pattern.search(line) is None:
                    list = line.split(';')
                    if len(list) > 1:
                        #Make sure we have a config file before including
                        if list[4] != "":
                            if type == "remote":
#                                self.remote_list.append(list[0].translate(bytes.maketrans('',''),','))
                                self.remote_list.append(list[0])
                                self.remote_count = self.remote_count + 1
                            elif type == "transmitter":
                                self.transmitter_list.append(list[0])
#                                self.transmitter_list.append(list[0].translate(bytes.maketrans('',''),','))
                                self.transmitter_count = self.transmitter_count + 1
        except IOError:
            print("Error reading " + file + " to parse remote listing.")
        #if type == "remote":
        #    self.remote_list.append("Custom") 
        #    self.remote_count = self.remote_count + 1
        #elif type == "transmitter":
        #    self.transmitter_list.append("Custom") 
        #    self.transmitter_count = self.transmitter_count + 1

    def set_device(self,device,type):
        """Sets an active remote control or transmitter"""
        if type == "remote":
            count=self.remote_count
            list=self.remote_list
        elif type == "transmitter":
            count=self.transmitter_count
            list=self.transmitter_list
        else:
            print("Invalid device type: " + type)
            return

        found = False
        for item in range(count):
            if list[item] == device[type]:
                if type == "remote":
                    self.remote = list[item]
                elif type == "transmitter":
                    self.transmitter = list[item]
                found = True
                break
        if device[type] == "Custom":
            if type == "remote":
                self.remote=device[type]
                self.remote_modules=device["remote_modules"]
                self.remote_driver=device["remote_driver"]
                self.remote_device=device["remote_device"]
                self.remote_lircd_conf=device["remote_lircd_conf"]
            elif type == "transmitter":
                self.transmitter=device[type]
                self.transmitter_modules=device["transmitter_modules"]
                self.transmitter_driver=device["transmitter_driver"]
                self.transmitter_device=device["transmitter_device"]
                self.transmitter_lircd_conf=device["transmitter_lircd_conf"]

    def get_device_dictionary(self,type):
        """Returns a dictionary of all the info for a given device type"""
        if type == "remote":
            return {"remote":self.remote, "remote_modules":self.remote_modules, "remote_device": self.remote_device, "remote_driver":self.remote_driver, "remote_lircd_conf":self.remote_lircd_conf}
        elif type == "transmitter":
            return {"transmitter":self.transmitter, "transmitter_modules":self.transmitter_modules, "transmitter_device": self.transmitter_device, "transmitter_driver":self.transmitter_driver, "transmitter_lircd_conf":self.transmitter_lircd_conf}

    def get_possible_devices(self,type):
        """Returns all the possible remotes or transmitters"""
        if type == "remote":
            return self.remote_list
        elif type == "transmitter":
            return self.transmitter_list
        else:
            print("Invalid type: " + type)
            return None

    def read_hardware_conf(self,file=HARDWARE_CONF):
        """Reads in a lirc hardware configuration"""
        try:
            in_f=open(file)
            remote_pattern=re.compile("^REMOTE=")
            found_remote=False
            remote_modules_pattern=re.compile("^REMOTE_MODULES=")
            remote_device_pattern=re.compile("^REMOTE_DEVICE=")
            remote_driver_pattern=re.compile("^REMOTE_DRIVER=")
            remote_lircd_conf_pattern=re.compile("^REMOTE_LIRCD_CONF=")
            transmitter_pattern=re.compile("^TRANSMITTER=")
            found_transmitter=False
            transmitter_modules_pattern=re.compile("^TRANSMITTER_MODULES=")
            transmitter_device_pattern=re.compile("^TRANSMITTER_DEVICE=")
            transmitter_driver_pattern=re.compile("^TRANSMITTER_DRIVER=")
            transmitter_lircd_conf_pattern=re.compile("^TRANSMITTER_LIRCD_CONF=")

            for line in in_f:
                if remote_pattern.search(line) and not found_remote:
                    for item in range(self.remote_count):
                        if (not found_remote) and (self.remote_list[item] == line.split('"')[1]):
                            found_remote = True
                            self.remote = line.split('"')[1]
                elif remote_modules_pattern.search(line):
                    self.remote_modules = line.split('"')[1]
                elif remote_device_pattern.search(line):
                    self.remote_device = line.split('"')[1]
                elif remote_driver_pattern.search(line):
                    self.remote_driver = line.split('"')[1]
                elif remote_lircd_conf_pattern.search(line):
                    self.remote_lircd_conf = line.split('"')[1]
                elif transmitter_pattern.search(line) and not found_transmitter:
                    for item in range(self.transmitter_count):
                        if (not found_transmitter) and (self.transmitter_list[item] == line.split('"')[1]):
                            found_transmitter = True
                            self.transmitter = line.split('"')[1]
                elif transmitter_modules_pattern.search(line):
                    self.transmitter_modules = line.split('"')[1]
                elif transmitter_device_pattern.search(line):
                    self.transmitter_device = line.split('"')[1]
                elif transmitter_driver_pattern.search(line):
                    self.transmitter_driver = line.split('"')[1]
                elif transmitter_lircd_conf_pattern.search(line):
                    self.transmitter_lircd_conf = line.split('"')[1]
            if not found_remote:
                self.remote=""
            if not found_transmitter:
                self.transmitter=""
            in_f.close()
        except IOError:
            print("Error opening " + file)

    def write_hardware_conf(self,file=HARDWARE_CONF):
        """Writes out a hardware.conf"""
        patternline="^REMOTE="
        patternline+="|^TRANSMITTER="
        patternline+="|^FORCE_NONINTERACTIVE_RECONFIGURATION="
        #If we are known to be a custom remote, then write out other stuff too
        if self.remote == "Custom":
            custom_remote = True
            patternline+="|^REMOTE_MODULES="
            patternline+="|^REMOTE_DRIVER="
            patternline+="|^REMOTE_LIRCD_CONF="
        else:
            custom_remote = False
        if self.transmitter == "Custom":
            custom_transmitter = True
            patternline+="|^TRANSMITTER_MODULES="
            patternline+="|^TRANSMITTER_DRIVER="
            patternline+="|^TRANSMITTER_LIRCD_CONF="
        else:
            custom_transmitter = False
        pattern=re.compile(patternline)
        found_remote=False
        found_transmitter=False
        lines = None
        try:
            in_f = open(file)
            lines = in_f.readlines()
            in_f.close()
        except IOError:
            print("Unable to find " + file)
        out_f = open(file, 'w')
        for line in lines:
            if pattern.search(line) is None:
                out_f.write(line)
                continue
            else:
                if not found_remote and re.compile("^REMOTE=").search(line):
                    out_f.write("REMOTE=\"" + self.remote + "\"\n")
                    found_remote = True
                    continue
                elif not found_transmitter and re.compile("^TRANSMITTER=").search(line):
                    out_f.write("TRANSMITTER=\"" + self.transmitter + "\"\n")
                    found_transmitter = True
                    continue
                if re.compile("^FORCE_NONINTERACTIVE_RECONFIGURATION=").search(line):
                    #a reconfigure of lirc will be necessary after writing this hardware conf
                    out_f.write("FORCE_NONINTERACTIVE_RECONFIGURATION=\"" + "true" + "\"\n")
                if custom_remote:
                    if re.compile("^REMOTE_MODULES=").search(line):
                        out_f.write("REMOTE_MODULES=\"" + self.remote_modules + "\"\n")
                        continue
                    elif re.compile("^REMOTE_DRIVER=").search(line):
                        out_f.write("REMOTE_DRIVER=\"" + self.remote_driver + "\"\n")
                        continue
                    elif re.compile("^REMOTE_DEVICE=").search(line):
                        out_f.write("REMOTE_DEVICE=\"" + self.remote_device + "\"\n")
                        continue
                    elif re.compile("^REMOTE_LIRCD_CONF=").search(line):
                        out_f.write("REMOTE_LIRCD_CONF=\"" + self.remote_lircd_conf + "\"\n")
                        continue
                if custom_transmitter:
                    if re.compile("^TRANSMITTER_MODULES=").search(line):
                        out_f.write("TRANSMITTER_MODULES=\"" + self.transmitter_modules + "\"\n")
                        continue
                    elif re.compile("^TRANSMITTER_DRIVER=").search(line):
                        out_f.write("TRANSMITTER_DRIVER=\"" + self.transmitter_driver + "\"\n")
                        continue
                    elif re.compile("^TRANSMITTER_DEVICE=").search(line):
                        out_f.write("TRANSMITTER_DEVICE=\"" + self.transmitter_device + "\"\n")
                        continue
                    elif re.compile("^TRANSMITTER_LIRCD_CONF=").search(line):
                        out_f.write("TRANSMITTER_LIRCD_CONF=\"" + self.transmitter_lircd_conf + "\"\n")
                        continue
        if not found_remote:
            out_f.write("REMOTE=\"" + self.remote + "\"\n")
        if not found_transmitter:
            out_f.write("TRANSMITTER=\"" + self.transmitter + "\"\n")
        out_f.close()

    def reconfigure_lirc(self,interactive=False):
        """Restarts & Reconfigures LIRC"""
        if interactive:
            os.putenv('DEBIAN_FRONTEND','gnome')
        else:
            os.putenv('DEBIAN_FRONTEND','noninteractive')
        lirc_command = subprocess.Popen(["/usr/sbin/dpkg-reconfigure", "lirc"],stdout=subprocess.PIPE).communicate()[0]
        print(lirc_command)

    def query_advanced(self,remote_hwdb_file=REMOTE_HWDB,transmitter_hwdb_file=TRANSMITTER_HWDB):
        """Determine if this remote requires interactive configuration"""

        advanced_driver_pattern=re.compile("devinput")
        advanced_modules_pattern=re.compile("serial")

        remote_hwdb = open(remote_hwdb_file).readlines()
        transmitter_hwdb = open(transmitter_hwdb_file).readlines()

        #Filter out uncessary lines
        filter = "^\#|^\["
        pattern = re.compile(filter)
        #search advanced remotes
        for line in remote_hwdb:
            if pattern.search(line) is None:
                list = line.split(";")
                if len(list) > 1 and list[0].translate(bytes.maketrans('',''),',') == self.remote:
                    if advanced_driver_pattern.search(list[1]) or advanced_modules_pattern.search(list[2]):
                        return True
        #search advanced transmitters
        for line in transmitter_hwdb:
            if pattern.search(line) is None:
                list = line.split(";")
                if len(list) > 1 and list[0].translate(bytes.maketrans('',''),',') == self.remote:
                    if advanced_driver_pattern.search(list[1]) or advanced_modules_pattern.search(list[2]):
                        return True
        return False
    def create_lircrc(self,file=LIRCD_CONF,change_permissions=True,irexec=False,irexec_only=False):
        """Generates lircrc files for all possible applications"""

        command = "mythbuntu-lircrc-generator"

        if file != LIRCD_CONF:
            command = command + " -L " + file
        
        if irexec == True:
            command = command + " --irexec 1"

        if irexec_only == True:
            command = command + " --mythtv 0 --mplayer 0 --totem 0 --elisa 0 --xine 0 --vlc 0 --xmame 0 --xmess 0"

        if change_permissions:
            uid = os.getenv('SUDO_UID')
            if uid != None:
                in_f=open("/etc/passwd")
                for line in in_f:
                    fields = line.split(":")
                    if fields[2] == uid:
                        command = "su " + fields[0] + " -c " + command
        os.system(command)



# -*- coding: utf-8 -*-
#
# «vnc» - libmythbuntu class for adding/removing VNC from configuration
#
# This script:
# Copyright (C) 2007-2008, Mario Limonciello, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import sys
import shutil

#import xorgconfig
import xkit
from xkit import xutils

#Script Directory
SCRIPTDIR = '/usr/share/mythbuntu-common/scripts'

class VNCHandler:
    """Used to properly enable VNC in a target configuration"""

    def __init__(self,root=''):
        self.add_modules = ["vnc"]
        self.add_screen = [ ['SecurityTypes', 'VncAuth'], ['UserPasswdVerifier', 'VncAuth'], ['PasswordFile', '/root/.vnc/passwd']]
        self.root = root
        
        self.xorg_conf_path = '/etc/X11/xorg.conf'
        
        try:
            self.xorg_conf = xutils.XUtils(self.xorg_conf_path)
        except(IOError, xkit.xorgparser.ParseException):#if xorg.conf is missing or broken
            self.xorg_conf = xutils.XUtils()

    def create_password(self,password):
        home = os.environ['HOME']
        if not os.path.exists(home + "/.vnc"):
            os.mkdir(home + "/.vnc")
        elif os.path.exists(home + "/.vnc/passwd"):
            print("Copying ~/.vnc/passwd to ~/.vnc/passwd.old")
            shutil.copy(home + "/.vnc/passwd",home + "/.vnc/passwd.old")

        if not isinstance(password, bytes):
            raise ValueError('Password should be passed as bytes')
        passpadd = (password + b'\x00' * 8)[:8]
        strkey = bytes(d3des.vnckey)
        ekey = d3des.deskey(strkey, False)
        ctext = d3des.desfunc(passpadd, ekey)
        with open(home + "/.vnc/passwd", 'wb') as f:
            f.write(ctext)

    def toggle_xorg(self,enable):
        """Adds necessary lines for enabling VNC upon the next boot"""

        # backup the current xorg.conf
        open(os.path.join(self.root + self.xorg_conf_path + ".old"), "w").write(open(self.root + self.xorg_conf_path).read())
        
        
        #Revert any existing settings first
        have_modules = len(self.xorg_conf.globaldict["Module"]) > 0
        have_screens = len(self.xorg_conf.globaldict["Screen"]) > 0
              
        if have_modules:
            for m in self.add_modules:
                self.xorg_conf.removeOption("Module", "Load", value=m, position=0)
        
        if have_screens:
            for item in self.add_screen:
                self.xorg_conf.removeOption("Screen", item[0], position=0)
        

        #Now re-enable if we want to
        if enable:
            if self.add_modules:
                if not have_modules:
                    self.xorg_conf.makeSection("Module")
                for m in self.add_modules:
                    self.xorg_conf.addOption("Module", "Load", m, position=0)
            
            if not have_screens:
                self.xorg_conf.makeSection("Screen", identifier="Default Screen")
            for item in self.add_screen:
                self.xorg_conf.addOption("Screen", item[0], item[1], optiontype="Option", position=0)
        
        self.xorg_conf.writeFile(self.root + self.xorg_conf_path)
        



## -*- coding: utf-8 -*-
#
# «remote» - MCC Infrared device selection plugin
#
# Copyright (C) 2010, John Baab, for Mythbuntu
# Copyright (C) 2009, Mario Limonciello, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin

from mythbuntu_common.lirc import LircHandler
from gi.repository import Gtk
import os
import subprocess
import logging

class RemotePlugin(MCCPlugin):
    """A GUI configuration tool for LIRC"""

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "Infrared"
        information["icon"] = "gtk-media-record"
        information["ui"] = "tab_remote_control"
        MCCPlugin.__init__(self,information)

        self.count={}
        self.count["remote"]=0
        self.count["transmitter"]=0

    #Helper functions
    def populate_lirc(self):
        """Fills the lirc pages with the appropriate data"""
        def fill_array(list,possible_items):
            """Fills an individual array up"""
            count=0
            for item in possible_items:
                list.append([item])
                count = count + 1
            return count

        for type in ('remote','transmitter'):
            list = self.builder.get_object(type + '_list')
            model = list.get_model()
            self.count[type]=fill_array(model, \
                                        self.lirc.get_possible_devices(type))
            list.set_active_iter(model.get_iter(Gtk.TreePath([0,0])))

    def find_lircrc(self):
        """Determines if there is current a lircrc for the current user"""
        return os.path.exists(os.getenv('HOME') + '/.lircrc')

    #Inherited, overridden functions
    def captureState(self):
        """Determines the state of the items on managed by this plugin"""
        #Parent functionality for state capturing
        MCCPlugin.clearParentState(self)
        #If this is a first run of capturing, we might need to fill
        #in our remote_count
        self.state = {}
        for item in ['lirc','mythbuntu-lirc-generator']:
            self.state[item] = self.query_installed(item)

        #TODO, detect smartphone type somehow.
        self.state['type'] = 'native'

        if self.state['lirc']:
            self.state['type'] = 'lirc'
            if self.count["remote"] is 0:
                self.lirc=LircHandler()
                self.populate_lirc()
            #Read in our /etc/lirc/hardware.conf
            self.lirc.read_hardware_conf()

    def applyStateToGUI(self):
        """Takes the current state in formation and sets the GUI"""

        def match_object(saved_setting, list, count):
            """Attempts to match a saved object to a GTK list of objects"""
            if saved_setting is "Custom":
                return True
            elif saved_setting != "None":
                for item in range(count):
                    model = list.get_model()
                    iter = model.get_iter(Gtk.TreePath([item,0]))
                    list.set_active_iter(iter)
                    if model[iter][0] == saved_setting:
                        return True
            return False

        def load_custom(type,dictionary):
            """Loads a custom remote or transmitter into the GUI"""
            modules=self.builder.get_object(type + '_modules')
            device=self.builder.get_object(type + '_device')
            driver=self.builder.get_object(type + '_driver')
            conf=self.builder.get_object('browse_' + type + '_lircd_conf')
            modules.set_text(dictionary[type + "_modules"])
            driver.set_text(dictionary[type + "_driver"])
            device.set_text(dictionary[type + "_device"])
            conf.select_filename("/usr/share/lirc/" + type )
            if os.path.exists("/usr/share/lirc/" + type + '/' + dictionary[type + "_lircd_conf"]):
                conf.select_filename("/usr/share/lirc/" + type + '/' + dictionary[type + "_lircd_conf"])

        if self.state['type'] == 'lirc':
            self.smartphone_vbox.set_sensitive(False)
            self.lirc_vbox.set_sensitive(True)
            self.lirc_support.set_active(True)
            for type in ('remote','transmitter'):
                dict=self.lirc.get_device_dictionary(type)
                list=self.builder.get_object(type + '_list')
                checkbox=self.builder.get_object(type + 'control')
                found=match_object(dict[type],list,self.count[type])
                checkbox.set_active(found)
                self.toggle_ir(checkbox,True)

                #"Custom" support
                load_custom(type,dict)
        elif self.state['type'] == 'native':
            self.lirc_vbox.set_sensitive(False)
            self.smartphone_vbox.set_sensitive(False)
        else:
            self.lirc_vbox.set_sensitive(False)
            self.smartphone_vbox.set_sensitive(True)

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        def test_basic_changes(type):
            """Test for changes in a regular remote or transmitter"""
            old=self.lirc.get_device_dictionary(type)[type]
            list = self.builder.get_object(type + '_list')
            model = list.get_model()
            current = model[list.get_active_iter()][0]
            if old != current:
                self._markReconfigureRoot(type,current)
            return current

        def test_custom_changes(type):
            """Tests for changes in a custom remote or transmitter"""
            old={}
            new={}
            old['modules']=self.lirc.get_device_dictionary(type)[type + "_modules"]
            old['device']=self.lirc.get_device_dictionary(type)[type + "_device"]
            old['driver']=self.lirc.get_device_dictionary(type)[type + "_driver"]
            old['lircd_conf']=self.lirc.get_device_dictionary(type)[type + "_lircd_conf"]
            new['modules']=self.builder.get_object(type + '_modules').get_text()
            new['device']=self.builder.get_object(type + '_device').get_text()
            new['driver']=self.builder.get_object(type + '_driver').get_text()
            new['lircd_conf']=self.builder.get_object('browse_' + type + '_lircd_conf').get_filename()
            for item in old:
                if old[item] != new[item]:
                    self._markReconfigureRoot(type + '_'+ item,new[item])

        #Prepare for state capturing
        MCCPlugin.clearParentState(self)

        #check for type changes
        if self.native_support.get_active():
            radio = 'native'
        elif self.lirc_support.get_active():
            radio = 'lirc'
        else:
            radio = 'smartphone'

        #figure out if we are installing lirc or removing it
        if self.state['type'] != radio:
                if radio == 'lirc' and not self.state['lirc']:
                    self._markInstall('lirc')
                elif self.state['lirc']:
                    self._markRemove('lirc')

        if self.state['lirc']:
            for type in ('remote','transmitter'):
                value = test_basic_changes(type)
                if value == 'Custom':
                    test_custom_changes(type)

            if self.generate_lircrc_checkbox.get_active():
                if not self.state['mythbuntu-lirc-generator']:
                    self._markInstall('mythbuntu-lirc-generator')
                if self.enable_irexec_checkbox.get_active():
                    self._markReconfigureUser("generate_lircrc_irexec", True)
                else:
                    self._markReconfigureUser("generate_lircrc",True)
            elif self.enable_irexec_checkbox.get_active():
                if not self.state['mythbuntu-lirc-generator']:
                    self._markInstall('mythbuntu-lirc-generator')
                self._markReconfigureUser("generate_lircrc_irexec_only", True)

    def toggle_ir(self,widget,internal=False):
        """Callback used for any widgets in the remote plugin"""

        def toggle_active_section(widget,type):
            """Turns on or off an IR remote"""
            hbox=self.builder.get_object(type + '_hbox')
            list=self.builder.get_object(type + '_list')
            hbox.set_sensitive(widget.get_active())
            if widget.get_active():
                if list.get_active() == 0:
                    list.set_active(1)
            else:
                list.set_active(0)

        def toggle_active_object_selection(widget,type):
            """Toggles the active device selection and related items"""
            control=self.builder.get_object(type + 'control')
            list=self.builder.get_object(type + '_list')
            driver=self.builder.get_object(type + '_driver_hbox')
            modules=self.builder.get_object(type + '_modules_hbox')
            device=self.builder.get_object(type + '_device_hbox')
            config=self.builder.get_object(type + '_configuration_hbox')

            if list.get_model()[list.get_active_iter()][0] == "Custom":
                custom = True
            else:
                custom = False
                if list.get_active() == 0:
                    control.set_active(False)
                    return False

            for item in (driver, modules, device, config):
                if custom:
                    item.show()
                else:
                    item.hide()
            return True

        if internal and self.find_lircrc():
            self.generate_lircrc_checkbox.set_active(False)
            self.enable_irexec_checkbox.set_active(False)

        if widget is not None:
            #toggle types on the page
            if type(widget) is Gtk.RadioButton:
                #they all have callbacks, only use the active one
                if widget.get_active():
                    name = widget.get_name()
                    if name == 'native_support':
                        self.smartphone_vbox.set_sensitive(False)
                        self.lirc_vbox.set_sensitive(False)
                    elif name == 'smartphone_support':
                        self.smartphone_vbox.set_sensitive(True)
                        self.lirc_vbox.set_sensitive(False)
                    #the lirc package needs to be installed to show this
                    elif self.state['lirc']:
                        self.smartphone_vbox.set_sensitive(False)
                        self.lirc_vbox.set_sensitive(True)

            #turn on/off IR remote
            elif widget.get_name() == 'remotecontrol':
                toggle_active_section(widget,"remote")

            #turn on/off IR transmitter
            elif widget.get_name() == "transmittercontrol":
                toggle_active_section(widget,"transmitter")

            #if our selected remote itself changed
            elif widget.get_name() == 'remote_list':
                if toggle_active_object_selection(widget,"remote"):
                    self.generate_lircrc_checkbox.set_active(True)
                else:
                    self.generate_lircrc_checkbox.set_active(False)

            #if our selected transmitter itself changed
            elif widget.get_name() == 'transmitter_list':
                toggle_active_object_selection(widget,"transmitter")

    def user_scripted_changes(self,reconfigure):
        """Local changes that can be performed by the user account.
           This function will be ran by the frontend."""
        for item in reconfigure:
            if item == "generate_lircrc":
                self.lirc.create_lircrc(change_permissions=False)
            elif item == "generate_lircrc_irexec":
                self.lirc.create_lircrc(change_permissions=False,irexec=True)
            elif item == "generate_lircrc_irexec_only":
                self.lirc.create_lircrc(change_permissions=False,irexec=True,irexec_only=True)

    def root_scripted_changes(self,reconfigure):
        """System-wide changes that need root access to be applied.
           This function is ran by the dbus backend"""
        lirc=LircHandler()
        found = False
        for item in reconfigure:
            if item == "remote" or item == "transmitter":
                logging.debug("Setting item %s to %s." % (item,reconfigure[item]))
                lirc.set_device({item:reconfigure[item]},item)
                found = True
        if found:
            logging.debug("Writing out /etc/lirc/hardware.conf")
            lirc.write_hardware_conf()
            logging.debug("Reconfiguring LIRC package")
            lirc.reconfigure_lirc(interactive=False)
        return found



# -*- coding: utf-8 -*-
#
# installer - Functions for use in Mythbuntu ubiquity plugins
#
# Copyright (C) 2010, Mario Limonciello, for Mythbuntu
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from ubiquity.plugin import *
from ubiquity.misc import create_bool
from gi.repository import Gtk

class MythPageGtk(PluginUI):
    def __init__(self, controller, *args, **kwargs):
        def add_widget(plugin, widget):
            """Make a widget callable by a plugin."""
            if not isinstance(widget, Gtk.Widget):
                return
            widget.set_name(Gtk.Buildable.get_name(widget))
            setattr(plugin, widget.get_name(), widget)
            if isinstance(widget, Gtk.Label):
                widget.set_property('can-focus', False)

        self.controller=controller
        try:
            builder = Gtk.Builder()
            builder.add_from_file('/usr/share/ubiquity/gtk/%s.ui' % self.ui_file)
            builder.connect_signals(self)
            for widget in builder.get_objects():
                add_widget(self, widget)
            self.plugin_widgets = builder.get_object(self.ui_file)
            self.controller.add_builder(builder)
        except Exception as e:
            self.debug('Could not create mythbuntu page: %s', e)
            self.plugin_widgets = None

def preseed_list(lists,names,value):
    """Helper function for preseeding dictionary based lists"""
    new_value = create_bool(value)
    for list in lists:
        for item in list:
            for name in names.split():
                if item == name:
                    #be careful what type of item we are dealing with
                    if type(list[item]) == Gtk.CheckButton:
                        list[item].set_active(new_value)
                    elif type(list[item]) == Gtk.Entry:
                        list[item].set_text(new_value)
                    elif type(list[item]) == Gtk.ComboBox:
                        model = list[item].get_model()
                        for iteration in range(len(list[item]),0):
                            list[item].set_active(iteration)
                            iterator = list[item].get_active_iter()
                            active = model.get_value(iterator, 0)
                            if active == new_value:
                                break
                    else:
                        list[item].set_active_text(new_value)

def build_static_list(lists):
    """Creates a flat list"""
    total_list= {}
    for list in lists:
        for item in list:
            if type(list[item]) == str:
                total_list[item]=list[item]
            elif type(list[item]) == Gtk.CheckButton:
                total_list[item]=list[item].get_active()
            elif type(list[item]) == Gtk.Entry:
                total_list[item]=list[item].get_text()
            else:
                model = list[item].get_model()
                row = list[item].get_selection().get_selected_rows()[1]
                iterator = model.get_iter(row[0])
                active = model.get_value(iterator, 0)
                total_list[item]=active
    return total_list



# -*- coding: utf-8 -*-
#
# «dictionaries» - Manage list of all widget -> package matching
#
# This script:
# Copyright (C) 2008, Mario Limonciello, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import string
import subprocess

####################
# Type dictionaries: different installation types possible
def get_install_type_dictionary(self):
    list= {
        "Master Backend/Frontend" : self.master_be_fe,                         \
        "Slave Backend/Frontend"  : self.slave_be_fe,                          \
        "Master Backend"          : self.master_be,                            \
        "Slave Backend"           : self.slave_be,                             \
        "Frontend"                : self.fe,                                   \
        "Set Top Box"             : self.stb }
    return list

####################
#Plugin dictionaries: these are for managing a list of all possible plugins
####################
def get_frontend_plugin_dictionary(self):
    list = {
        "mytharchive": self.mytharchive_checkbox,                              \
        "mythbrowser": self.mythbrowser_checkbox,                              \
        "mythgame": self.mythgame_checkbox,                                    \
        "mythgallery": self.mythgallery_checkbox,                              \
        "mythmusic": self.mythmusic_checkbox,                                  \
        "mythnews": self.mythnews_checkbox,                                    \
        "mythnetvision": self.mythnetvision_checkbox,                          \
        "mythweather": self.mythweather_checkbox }
    return list

def get_backend_plugin_dictionary(self):
    return { "mythweb": self.mythweb_checkbox }

####################
#Other dictionaries: these are for managing a list of all other apps
####################
def get_role_dictionary(self):
    list = {
        "mythtv-backend-master": self.primary_backend_radio,                   \
        "mythtv-backend":        self.secondary_backend_radio,                 \
        "mythtv-frontend":       self.frontend_radio }
    return list

def get_desktop_dictionary(self):
    list = {
        "ubuntu-desktop": self.ubuntu_desk_check,                              \
        "xubuntu-desktop": self.xubuntu_desk_check,                            \
        "kubuntu-desktop": self.kubuntu_desk_check }
    return list

def get_media_app_dictionary(self):
    list = {
        "mplayer": self.mplayer_checkbox,                                      \
        "xine-ui": self.xine_checkbox,                                         \
        "vlc":  self.vlc_checkbox }
    return list

def get_nonfree_dictionary(self):
    list = {
#        "medibuntu-keyring": self.medibuntu_keyring,                           \
        "libdvdcss2": self.enable_libdvdcss2 }
    return list

def get_services_dictionary(self,sql_object=None):
    list = {
        "x11vnc": self.enablevnc,                                              \
        "samba": self.enablesamba,                                             \
        "nfs-kernel-server": self.enablenfs,                                   \
        "openssh-server": self.enablessh,                                      \
        "mysql-server": sql_object }
    return list

def get_diskless_dictionary(self):
    list = {
        "mythbuntu-diskless-server": self.diskless_server_install,             \
        "dhcp3-server": self.diskless_server_dhcp }
    return list

def get_graphics_dictionary():
    list = {}
    ##AMD graphics detection
    #output=string.split(subprocess.Popen(['lspci'],stdout=subprocess.PIPE).communicate()[0],'\n')
    #for line in output:
    #    if line and 'VGA' in line and ('AMD' in line or 'ATI' in line):
    #        list["AMD Graphics"]="fglrx"
    #        break

    #NVIDIA Graphics detection
    try:
        from NvidiaDetector.nvidiadetector import NvidiaDetection
    except ImportError:
        return list

    nv = NvidiaDetection().selectDriver()
    if nv is not None:
        list["NVIDIA Graphics"]=nv
    return list

def get_tweak_dictionary(self):
    list = {
        "/etc/mysql/conf.d/mythtv-tweaks.cnf": self.enable_mysql_tweaks, \
#        "/etc/cron.daily/mythtv-xfs-defrag": self.enable_nightly_defrag, \
        "/etc/cron.daily/optimize_mythdb": self.enable_mysql_repair}
    return list



# -*- coding: utf-8 -*-
#
# «mysql» - mythbuntu class for mysql mangling
#
# This script:
# Copyright (C) 2007-2010, Mario Limonciello, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import shutil
import os
import subprocess
import re
import string
import xml.dom.minidom


#pymysql isn't available yet.
#for now don't fail on import
try:
    import pymysql
except:
    pass

class MySQLHandler:
    """MySQL configuration, mangling, and activation class"""

    def __init__(self,config={"user":"mythtv","password":"mythtv","server":"localhost","database":"mythconverg","securitypin":"0000"}):
        self.user=config["user"]
        self.password=config["password"]
        self.server=config["server"]
        self.database=config["database"]
        self.securitypin=config["securitypin"]

    def _config_xml(self, file=None):
        """Reads in and parses and writes a config.xml from a number of places:
           1) /etc/mythtv/config.xml
           2) /usr/share/mythtv/config.xml
           Returns a dictionary filled with references that can be parsed or processed
        """
        dictionary={"doc": None, 'DBHostName': None, 'DBUserName': None, 'DBPassword': None, 'DBName': None, 'SecurityPin' : None}
        for file in [file, '/etc/mythtv/config.xml', '/usr/share/mythtv/config.xml' ]:
            if file and os.path.exists(file):
                dictionary["doc"]=xml.dom.minidom.parse(file)
                for tag in dictionary:
                    elements=dictionary["doc"].getElementsByTagName(tag)
                    if elements:
                        dictionary[tag]=elements[0]
                    elif tag != "doc":
                        dictionary[tag]=dictionary["doc"].createElement(tag)
                break
        return dictionary

    def write_xml(self,file):
        """Writes XML to a file"""
        dict=self._config_xml(file)
        if dict['DBHostName'].hasChildNodes():
            dict['DBHostName'].childNodes[0].replaceWholeText(self.server)
        else:
            node = dom.xml.minidom.createTextNode(self.server)
            dict['DBHostName'].appendChild(node)
        if dict['DBUserName'].hasChildNodes():
            dict['DBUserName'].childNodes[0].replaceWholeText(self.user)
        else:
            node = dom.xml.minidom.createTextNode(self.user)
            dict['DBUserName'].appendChild(node)
        if dict['DBPassword'].hasChildNodes():
            dict['DBPassword'].childNodes[0].replaceWholeText(self.password)
        else:
            node = dom.xml.minidom.createTextNode(self.password)
            dict['DBPassword'].appendChild(node)
        if dict['DBName'].hasChildNodes():
            dict['DBName'].childNodes[0].replaceWholeText(self.database)
        else:
            node = dom.xml.minidom.createTextNode(self.database)
            dict['DBName'].appendChild(node)
        if dict['SecurityPin'].hasChildNodes():
            dict['SecurityPin'].childNodes[0].replaceWholeText(self.securitypin)
        else:
            node = dom.xml.minidom.createTextNode(self.securitypin)
            dict['SecurityPin'].appendChild(node)

        dict["doc"].writexml(open(file, 'w'))

    def read_xml(self,file):
        dict=self._config_xml(file)
        if dict['DBHostName'].hasChildNodes():
            self.server = dict['DBHostName'].childNodes[0].data
        if dict['DBUserName'].hasChildNodes():
            self.user = dict['DBUserName'].childNodes[0].data
        if dict['DBPassword'].hasChildNodes():
            self.password = dict['DBPassword'].childNodes[0].data
        if dict['DBName'].hasChildNodes():
            self.database = dict['DBName'].childNodes[0].data
        if dict['SecurityPin'].hasChildNodes():
            self.securitypin = dict['SecurityPin'].childNodes[0].data

    def toggle_mysql_service_config(self,enable):
        """Enables and disables the mysql service on all interfaces"""
        if not os.path.exists("/etc/mysql/conf.d"):
            os.mkdir("/etc/mysql/conf.d")
        lines = None
        out_f = None
        found = False
        pattern = re.compile("^bind-address|^#bind-address")
        try:
            in_f = open("/etc/mysql/conf.d/mythtv.cnf")
            lines = in_f.readlines()
            in_f.close()
            out_f=open("/etc/mysql/conf.d/mythtv.cnf","w")
            for line in lines:
                if pattern.search(line) is None:
                    out_f.write(line)
                elif not found:
                    if enable:
                        out_f.write("bind-address=0.0.0.0\n")
                    else:
                        out_f.write("#bind-address=0.0.0.0\n")
                    found = True
            if not found:
                if enable:
                    out_f.write("bind-address=0.0.0.0\n")
                else:
                    out_f.write("#bind-address=0.0.0.0\n")
        except IOError:
            print("/etc/mysql/conf.d/mythtv.cnf not found")
            out_f=open("/etc/mysql/conf.d/mythtv.cnf","w")
            out_f.write("[mysqld]\n")
            if enable:
                out_f.write("bind-address=0.0.0.0\n")
            else:
                out_f.write("#bind-address=0.0.0.0\n")
        out_f.close()

    def restart_mysql_service(self):
        """Restarts MySQL service"""
        start_mysql = subprocess.Popen(["/usr/sbin/invoke-rc.d", "mysql", "restart"],stdout=subprocess.PIPE).communicate()[0]
        print(start_mysql)

    def update_config(self,config):
        """Sets up a new configuration based on the dict {user,pass,server,db,securitypin}"""
        self.user=config["user"]
        self.password=config["password"]
        self.server=config["server"]
        self.database=config["database"]
        self.securitypin=config["securitypin"]

    def get_config(self):
        """Returns our currently stored configuration"""
        return {"user":self.user,"password":self.password,"server":self.server,"database":self.database,"securitypin":self.securitypin}

    def reset_user_password(self,admin_pass,password):
        """Resets a user's password if it was forgotten"""
        commands = ["UPDATE user SET Password=PASSWORD('"+password+"') WHERE user=mythtv')",
        "FLUSH PRIVILEGES"]
        if self.run_mysql_commands(commands,mysql_user="root"):
            return "Successful"
        else:
            return "Failure"

    def run_mysql_commands(self,commands,mysql_user=None):
        """Runs mysql command(s) and returns the response"""
        if mysql_user is None:
            mysql_user = self.user
        try:
            db = pymysql.Connect(host=self.server, user=mysql_user, passwd=self.password,database=self.database)
            cursor = db.cursor()
            if type(commands) is list:
                for command in commands:
                    cursor.execute(command)
                    result = cursor.fetchone()
            elif type(commands) is str:
                result = cursor.execute(commands)
            else:
                print("Unknown type")
            cursor.close()
            db.close()
        except:
            result = False
        return result

    def do_connection_test(self, pin):
        """Tests to make sure that the backend is accessible"""
        try:
            import MythTV
        except ImportError:
            return False
        args = { 'SecurityPin' : pin }
        #figure out whether 0.24+ or 0.23
        try:
            method = getattr(MythTV, "MythDBBase")
        except AttributeError:
            method = getattr(MythTV, "MythDB")
        try:
            db_base = method(args=args)
        except Exception as e:
            return str(e)



## -*- coding: utf-8 -*-
#
# «Mythbuntu Repos» - A Plugin for adding Mythbuntu Repos. (ie. -testing and
# Auto-Builds)
#
# Copyright (C) 2009, Thomas Mashos, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin
import os
import re
import subprocess
import urllib.request, urllib.error, urllib.parse
import shutil
import subprocess
import configparser
#from bzrlib.plugin import load_plugins
#from bzrlib import branch, tree
import time

class MythbuntuReposPlugin(MCCPlugin):
    """A Plugin for adding Mythbuntu Repos"""
    #
    #Load GUI & Calculate Changes
    #

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "Repositories"
        information["icon"] = "emblem-downloads"
        information["ui"] = "tab_repos"
        MCCPlugin.__init__(self, information)
        self.CONFIGFILE = "/etc/default/mythbuntu-repos"
        self.USERHOME = os.path.expanduser("~")
        if not os.path.isdir(self.USERHOME+"/.mythbuntu"):
            os.mkdir(self.USERHOME+"/.mythbuntu")
        if not os.path.isfile(self.USERHOME+"/.mythbuntu/repos.db"):
            try:
                #pass
                self.downloadFile()
            except:
                pass
                #shutil.copy("/usr/share/mythbuntu/repos.db", self.USERHOME+"/.mythbuntu/")
        self.config = configparser.ConfigParser()

    #Set mythtv versions
    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""
        self.versions = []
        for line in open("/etc/lsb-release"):
            if "DISTRIB_CODENAME" in line:
                line = line.strip("\n")
                throwaway, distro = line.split("=")
        NumRepos = 0

        if os.path.isfile(self.USERHOME+"/.mythbuntu/repos.db"):
            self.versions = []
            for line in open(self.USERHOME+"/.mythbuntu/repos.db"):
                if distro in line:
                    line = line.strip("\n")
                    release, version = line.split("\t")
                    if not version in self.versions:
                        self.versions.append(version)
                    NumRepos += 1
                elif "MYTHTV_RELEASE" in line:
                    line = line.strip("\n")
                    VerName, self.CurVer = line.split("\t")
                elif "TRUNKPASS" in line:
                    line = line.strip("\n")
                    discard, self.TRUNKPASS = line.split("\t")
            self.download_repo_db_label.hide()
            self.mythtv_updates_alignment.show()
            self.mythtv_updates_ckbox_alignment.show()
            self.mythtv_updates_label.show()
            self.mythtv_updates_checkbox.show()
            self.repobox.show()
            self.hseparator3.show()
            self.mythbuntu_updates_alignment.show()
            self.mythbuntu_updates_checkbox_alignment.show()
            self.hseparator4.show()
            self.xmltv_updates_alignment.show()
            self.xmltv_updates_checkbox_alignment.show()
            self.hseparator5.show()
            self.footer_alignment.show()
        else:
            self.versions.append('0')
            self.CurVer = '0'
            self.download_repo_db_label.show()
            self.mythtv_updates_alignment.hide()
            self.mythtv_updates_ckbox_alignment.hide()
            self.mythtv_updates_label.hide()
            self.mythtv_updates_checkbox.hide()
            self.repobox.hide()
            self.hseparator3.hide()
            self.mythbuntu_updates_alignment.hide()
            self.mythbuntu_updates_checkbox_alignment.hide()
            self.hseparator4.hide()
            self.xmltv_updates_alignment.hide()
            self.xmltv_updates_checkbox_alignment.hide()
            self.hseparator5.hide()
            self.footer_alignment.hide()
            self.trunk_block.hide()
            
        self.changes = {}
        self.repobox.get_model().clear()
        for item in self.versions:
            self.builder.get_object('repobox').append_text(item)
        if os.path.exists(self.CONFIGFILE):
            self.config.read(self.CONFIGFILE)
        try:
            self.changes['MythTVUpdatesActivated'] = self.config.getboolean("cfg", "ActivateMythTVUpdates")
        except:
            self.changes['MythTVUpdatesActivated'] = False
        try:
            self.changes['MythTVUpdatesRepo'] = self.config.get("cfg", "MythTVRepo")
        except:            
            self.changes['MythTVUpdatesRepo'] = self.versions[0]
        try:
            self.changes['ActivateMythbuntuUpdates'] = self.config.getboolean("cfg", "ActivateMythbuntuUpdates")
        except:
            self.changes['ActivateMythbuntuUpdates'] = False
        try:
            self.changes['ActivateXMLTVUpdates'] = self.config.getboolean("cfg", "ActivateXMLTVUpdates")
        except:
            self.changes['ActivateXMLTVUpdates'] = False

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""
        self.mythtv_updates_checkbox.set_active(self.changes['MythTVUpdatesActivated'])
        self.repobox.set_sensitive(self.changes['MythTVUpdatesActivated'])
        try:
            self.repobox.set_active(self.versions.index(self.changes['MythTVUpdatesRepo']))
        except ValueError:
            self.repobox.set_active(0)
        self.mythbuntu_updates_checkbox.set_active(self.changes['ActivateMythbuntuUpdates'])
        self.xmltv_updates_checkbox.set_active(self.changes['ActivateXMLTVUpdates'])
        self.trunk_pass_ok.hide()

    def on_mythtv_updates_checkbox_toggled(self, widget, data=None):
        """Show the repobox if this is checked"""
        widget_was_visible = self.mythtv_updates_checkbox.get_active()
        if widget_was_visible:
            self.repobox.set_sensitive(True)
        else:
            self.repobox.set_sensitive(False)

    def on_dev_password_entry_changed(self, widget, data=None):
        """Require a password for using trunk"""
        if self.dev_password_entry.get_text() == self.TRUNKPASS:
            self.trunk_pass_error.hide()
            self.trunk_pass_ok.show()
        else:
            self.trunk_pass_error.show()
            self.trunk_pass_ok.hide()

    def on_repobox_changed(self, widget, data=None):
        """Check the version to know if it's trunk"""
        if not self.repobox.get_active_text() == None:
            SELVER = self.convertVersion(self.repobox.get_active_text())
            if SELVER > self.CurVer:
                self.trunk_block.show()
            else:
                self.trunk_block.hide()

    def convertVersion(self, VERSION):
        """Remove the trailing .x if it exists"""
        if VERSION.endswith(".x"):
            VERSION = VERSION.strip(".x")
        return VERSION

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        MCCPlugin.clearParentState(self)
        SENDLIST = False
        SELVER = self.convertVersion(self.repobox.get_active_text())
        if self.mythtv_updates_checkbox.get_active() != self.changes['MythTVUpdatesActivated'] and self.mythtv_updates_checkbox.get_active() == False:
            self._markReconfigureRoot('MythTV-Updates-Activated', self.mythtv_updates_checkbox.get_active())
            SENDLIST = True
        if self.repobox.get_sensitive() == True:
            if self.repobox.get_active_text() != self.changes['MythTVUpdatesRepo'] or self.mythtv_updates_checkbox.get_active() != self.changes['MythTVUpdatesActivated']:
                if (SELVER > self.CurVer and self.dev_password_entry.get_text() == self.TRUNKPASS) or SELVER <= self.CurVer:
                    self._markReconfigureRoot('MythTV-Updates-Repo', self.repobox.get_active_text())
                    self._markReconfigureRoot('MythTV-Updates-Activated', self.mythtv_updates_checkbox.get_active())
                    SENDLIST = True
                elif self.mythtv_updates_checkbox.get_active() == False:
                    self._markReconfigureRoot('MythTV-Updates-Activated', self.mythtv_updates_checkbox.get_active())
        if self.mythbuntu_updates_checkbox.get_active() != self.changes['ActivateMythbuntuUpdates']:
            self._markReconfigureRoot('Mythbuntu-Updates-Activated', self.mythbuntu_updates_checkbox.get_active())
        if self.xmltv_updates_checkbox.get_active() != self.changes['ActivateXMLTVUpdates']:
            self._markReconfigureRoot('XMLTV-Updates-Activated', self.xmltv_updates_checkbox.get_active())
        if SENDLIST == True:
            self._markReconfigureRoot('Repo-list', self.versions)

    def refresh_button_clicked(self, widget, data=None):
        """Download a new db file if requested"""
        self.downloadFile()
        self.captureState()
        self.applyStateToGUI()

    def downloadFile(self):
        """Download files"""
        self.emit_progress("Refreshing available repos from server", 0)
        time.sleep(.5)
        url = 'http://download.mythbuntu.org/repos/repos.db'
        try:
            import urllib.request
            from urllib.error import HTTPError,URLError
            # Open the url
            f = urllib.request.urlopen(url)
            # Open our local file for writing
            local_repo_file = open(self.USERHOME+"/.mythbuntu/repos.db", "wb")
            #Write to our local file
            local_repo_file.write(f.read())
            local_repo_file.close()
            self.emit_progress("New DB file download finished", 100)
        #handle errors
        except HTTPError as e:
            print("HTTP Error:",e.code , url)
            self.emit_progress("HTTP Error: Failed to download new DB file", 0)
        except URLError as e:
            print("URL Error:",e.reason , url)
            self.emit_progress("URL Error: Failed to download new DB file", 0)
        time.sleep(2)
        #self.close_progress()

    #
    # Process selected activities
    #

    def root_scripted_changes(self, reconfigure):
        """System-wide changes that need root access to be applied.
           This function is ran by the dbus backend"""
        self.emit_progress("Opening config file", 10)
        if os.path.exists(self.CONFIGFILE):
            self.config.read("/etc/default/mythbuntu-repos")
        else:
            self.config.add_section("cfg")
        if "Repo-list" in reconfigure:
            self.emit_progress("Removing old repositories", 40)
            for item in reconfigure["Repo-list"]:
                if item.endswith(".x"):
                    item = item.strip(".x")
                    subprocess.call(["apt-add-repository", "-r", "-y", "ppa:mythbuntu/"+item])
        if "MythTV-Updates-Activated" in reconfigure:
            self.emit_progress("Configuring MythTV Updates repo", 50)
            if reconfigure["MythTV-Updates-Activated"]:
                repo = reconfigure["MythTV-Updates-Repo"]
                self.config.set("cfg", "ActivateMythTVUpdates", "True")
                self.config.set("cfg", "MythTVRepo", repo)
                if repo.endswith(".x"):
                    repo = repo.strip(".x")
                subprocess.call(["apt-add-repository", "-y", "ppa:mythbuntu/"+repo])
            else:
                self.config.set("cfg", "ActivateMythTVUpdates", "False")
        if "Mythbuntu-Updates-Activated" in reconfigure:
            self.emit_progress("Configuring Mythbuntu Updates repo", 60)
            if reconfigure["Mythbuntu-Updates-Activated"]:
                subprocess.call(["apt-add-repository", "-y", "ppa:mythbuntu/testing"])
                self.config.set("cfg", "ActivateMythbuntuUpdates", "True")
            else:
                subprocess.call(["apt-add-repository", "-r", "-y", "ppa:mythbuntu/testing"])
                self.config.set("cfg", "ActivateMythbuntuUpdates", "False")
        if "XMLTV-Updates-Activated" in reconfigure:
            self.emit_progress("Configuring XMLTV Updates repo", 70)
            if reconfigure["XMLTV-Updates-Activated"]:
                subprocess.call(["apt-add-repository", "-y", "ppa:mythbuntu/xmltv"])
                self.config.set("cfg", "ActivateXMLTVUpdates", "True")
            else:
                subprocess.call(["apt-add-repository", "-r", "-y", "ppa:mythbuntu/xmltv"])
                self.config.set("cfg", "ActivateXMLTVUpdates", "False")
        with open('/etc/default/mythbuntu-repos', 'w', encoding='utf8') as configfile:
            self.emit_progress("Writing config file", 80)
            self.config.write(configfile)
        self.emit_progress("Done configuring repositories", 100)
        time.sleep(2)



#!/usr/bin/python3
# mythbuntu-common install script
# Copyright (C) 2007-2009, Mario Limonciello <superm1@mythbuntu.org>
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from distutils.core import setup
from DistUtilsExtra.command import *

import glob, os.path

setup(
    name="mythbuntu-common",
    author="Mario Limonciello",
    author_email="superm1@mythbuntu.org",
    maintainer="Ubuntu MythTV Team",
    maintainer_email="ubuntu-mythtv@lists.ubuntu.com",
    url="http://www.mythbuntu.org",
    license="gpl",
    description="commonly used mythbuntu functions",
    packages=['mythbuntu_common'],
    data_files=[("share/mythbuntu/examples", glob.glob("examples/*")),
                ("share/mythbuntu/plugins/python", glob.glob("plugins/python/*")),
                ("share/mythbuntu/qr", glob.glob("qr/*")),
                ("share/mythbuntu/", glob.glob("*.db")),
                ("share/mythbuntu/plugins/ui", glob.glob("plugins/ui/*"))],
    scripts=[],
    cmdclass = { "build" : build_extra.build_extra,
        "build_i18n" : build_i18n.build_i18n,
        "clean" : clean_i18n.clean_i18n
        }
)



## -*- coding: utf-8 -*-
#
# «mysql_configuration» - MCC MysQL Related configuration plugin
#
# Copyright (C) 2009, Mario Limonciello, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin
import os
import re
import logging

from mythbuntu_common.mysql import MySQLHandler
from mythbuntu_common.dictionaries import get_tweak_dictionary

MYSQL_TXT='/etc/mythtv/mysql.txt'

MYTHTV_SETUP='/usr/bin/mythtv-setup'
OPTIMIZE_DATABASE="/usr/bin/x-terminal-emulator -e perl /usr/share/doc/mythtv-backend/contrib/optimize_mythdb.pl"

class MySQLConfigurationPlugin(MCCPlugin):
    """A configuration tool for MySQL Related Connectivity"""

    def __init__(self):
        #Initialize parent class
        information = {}
        information["name"] = "MySQL"
        information["icon"] = "gtk-network"
        information["ui"] = "tab_mysql_configuration"
        MCCPlugin.__init__(self,information)
        self.mysql=MySQLHandler()

    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""

        #Roles
        self.frontend=False
        self.mysql_service=False
        if self.query_installed('mythtv-backend-master'):
            #enable master backend and backend, disable frontend
            self.master=True
            self.backend=True
            if os.path.exists("/etc/mysql/conf.d/mythtv.cnf"):
                in_f=open("/etc/mysql/conf.d/mythtv.cnf")
                for line in in_f:
                    if re.compile("^bind-address").search(line):
                        self.mysql_service=True
                        break
                in_f.close()
        else:
            self.master=False
            self.backend=self.query_installed('mythtv-backend')

        if os.path.exists(os.path.join(os.environ['HOME'], '.mythtv', 'config.xml')):
            self.mysql.read_xml(os.path.join(os.environ['HOME'], '.mythtv', 'config.xml'))

        #Dictionaries
        self.dictionary_state={}
        list=get_tweak_dictionary(self)
        for item in list:
            self.dictionary_state[list[item]]=os.path.exists(item)

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""

        #Master
        if self.master:
            self.master_backend_vbox.show()
            if self.mysql_service:
                self.enablemysql.set_active(1)
            else:
                self.enablemysql.set_active(0)
        else:
            self.master_backend_vbox.hide()

        #Backend
        if self.backend:
            self.backend_vbox.show()
        else:
            self.backend_vbox.hide()

        #Connectivity
        config=self.mysql.get_config()
        self.security_entry.set_text(config["securitypin"])
        self.mysql_test_hbox.hide()

        #Dictionaries
        for item in self.dictionary_state:
            item.set_active(self.dictionary_state[item])

        self._incomplete=False

    def compareState(self):
        """Determines what items have been modified on this plugin"""
        #Prepare for state capturing
        MCCPlugin.clearParentState(self)

        list=get_tweak_dictionary(self)
        for item in list:
            if list[item].get_active() != self.dictionary_state[list[item]]:
                self._markReconfigureRoot(item,list[item].get_active())

        if self.master:
            if self.mysql_service and self.enablemysql.get_active_text() == "Disable":
                #disable service
                self._markReconfigureRoot("mysql_service",False)
            if not self.mysql_service and self.enablemysql.get_active_text() == "Enable":
                #enable service
                self._markReconfigureRoot("mysql_service",True)
        else:
            config=self.mysql.get_config()
            if self.security_entry.get_text() != config["securitypin"]:
                self._markReconfigureUser("securitypin",self.security_entry.get_text())


    def root_scripted_changes(self,reconfigure):
        """System-wide changes that need root access to be applied.
           This function is ran by the dbus backend"""
        for item in reconfigure:
            if item == "mysql_service":
                self.mysql.toggle_mysql_service_config(reconfigure[item])
                self.mysql.restart_mysql_service()
            elif item == "/etc/mysql/conf.d/mythtv-tweaks.cnf" or\
                 item == "/etc/cron.daily/optimize_mythdb":
                if reconfigure[item]:
                    import shutil
                    try:
                        if os.path.exists(item):
                            os.remove(item)
                        if item == "/etc/mysql/conf.d/mythtv-tweaks.cnf":
                            shutil.copy('/usr/share/mythbuntu/examples/mythtv-tweaks.dist',item)
                        else:
                            shutil.copy('/usr/share/doc/mythtv-backend/contrib/maintenance/optimize_mythdb.pl',item)
                            os.chmod(item,0o755)
                    except Exception as msg:
                        logging.warning("Exception when enabling %s, %s" % (item,msg))
                else:
                    try:
                        os.remove(item)
                    except Exception as msg:
                        logging.warning("Exception when disabling item %s, %s" % (item,msg))

    def user_scripted_changes(self, reconfigure):
        for item in reconfigure:
            if item == "securitypin":
                logging.debug("Updating MySQL config")
                #We don't actually have to modify this - if it was successful, it would have happened from the test.
                self.mysql.read_xml()
                if not os.path.exists(os.path.join(os.environ['HOME'],'.mythtv')):
                    os.makedirs(os.path.join(os.environ['HOME'],'.mythtv'))
                self.mysql.write_mysql_txt(os.path.join(os.environ['HOME'],'.mythtv','mysql.txt'))

    def do_connection_test(self,widget):
        """Performs a connectivity test to the backend's mysql server"""
        if widget is not None:
            self.mysql_test_hbox.show()
            result = self.mysql.do_connection_test(self.security_entry.get_text())
            if not result:
                self.pass_mysql.show()
                self.fail_mysql.hide()
                self._incomplete=False
                return
            self.pass_mysql.hide()
            self.fail_mysql.show()
            self._incomplete=True


    def launch_app(self,widget):
        """Launches an app defined in the glade file"""
        if widget.get_name() == 'mysql_tweak_button':
            MCCPlugin.launch_app(self,widget,OPTIMIZE_DATABASE)
        elif widget.get_name() == 'mythtv_setup_button':
            MCCPlugin.launch_app(self,widget,MYTHTV_SETUP)



## -*- coding: utf-8 -*-
#
# «services» - MCC Services enablement plugin
#
# Copyright (C) 2009, Mario Limonciello, for Mythbuntu
#
#
# Mythbuntu is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this application; if not, write to the Free Software Foundation, Inc., 51
# Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
##################################################################################

from MythbuntuControlCentre.plugin import MCCPlugin
from mythbuntu_common.dictionaries import get_services_dictionary
from mythbuntu_common.vnc import VNCHandler
import re
import shutil
import os
from gi.repository import Gtk

class MCCServices(MCCPlugin):
    """Services enablement plugin"""

    def __init__(self):
        information = {}
        information["name"] = "Services"
        information["icon"] = "gnome-settings"
        information["ui"] = "tab_services"
        MCCPlugin.__init__(self,information)
        self.vnc=VNCHandler()

    def captureState(self):
        """Determines the state of the items on managed by this plugin
           and stores it into the plugin's own internal structures"""
        #Most services meet this category
        self.dictionary_state={}
        list = get_services_dictionary(self)
        for item in list:
            if list[item] is not None:     
                self.dictionary_state[list[item]]=self.query_installed(item)

        #VNC password
        self.vnc_pass=''

    def applyStateToGUI(self):
        """Takes the current state information and sets the GUI
           for this plugin"""
        #from previous runs
        self.vnc_pass_hbox.hide()

        #Load the detected dictionary
        for item in self.dictionary_state:
            model = item.get_model()
            if len(model) > 2:
                iter = model.get_iter(Gtk.TreePath([2,0]))
                model.remove(iter)
            if self.dictionary_state[item]:
                item.set_active_iter(model.get_iter(Gtk.TreePath([1,0])))
                model.append(["Reconfigure"])
            else:
                item.set_active_iter(model.get_iter(Gtk.TreePath([0,0])))

        #Corner cases that don't need reconfiguring
        model = self.enablessh.get_model()
        if len(model) > 2:
            iter = model.get_iter(Gtk.TreePath([2,0]))
            model.remove(iter)


    def compareState(self):
        """Determines what items have been modified on this plugin"""
        #Prepare for state capturing
        MCCPlugin.clearParentState(self)

        list=get_services_dictionary(self)
        for item in list:
            if list[item] is not None and \
               list[item].get_active() != self.dictionary_state[list[item]]:
                #Installable items
                if list[item].get_active() == 1:
                    self._markInstall(item)
                #Removable items
                elif list[item].get_active() == 0:
                    self._markRemove(item)

                #Reconfigurable items:
                if list[item].get_active() > 0  and \
                   item != 'openssh-server'     and \
                   item != 'x11vnc':
                    self._markReconfigureRoot(item,list[item].get_active())

        #VNC password
        if self.vnc_pass != self.vnc_password.get_text():
            self._markReconfigureUser("x11vnc_password",self.vnc_password.get_text())

    def toggle_vnc(self,widget):
        """Called whenever VNC service is toggled"""
        if (widget is not None and widget.get_name() == 'enablevnc'):
            iteration=1
            if self.dictionary_state[self.enablevnc]:
                iteration = 2
            if widget.get_active() == iteration:
                self.vnc_pass_hbox.show()
                self.vnc_error_image.show()
                self._incomplete=True
            else:
                self.vnc_password.set_text("")
                self.vnc_pass_hbox.hide()
                self.vnc_error_image.hide()
                self._incomplete=False

    def toggle_vnc_password(self,widget):
        if (widget is not None and widget.get_name() == 'vnc_password'):
            password= widget.get_text().split(' ')[0]
            if len(password) >= 6:
                self.vnc_error_image.hide()
                self._incomplete=False
            else:
                self.vnc_error_image.show()
                self._incomplete=True

    def root_scripted_changes(self,reconfigure):
        """System-wide changes that need root access to be applied.
           This function is ran by the dbus backend"""
        for item in reconfigure:
            if item == "samba":
                if os.path.exists('/etc/samba/smb.conf'):
                    shutil.copy('/etc/samba/smb.conf','/etc/samba/smb.conf.dpkg-dist')
                shutil.copy('/usr/share/mythbuntu/examples/smb.conf.dist',
                            '/etc/samba/smb.conf')
            elif item == "nfs-kernel-server":
                if os.path.exists('/etc/exports'):
                    shutil.copy('/etc/exports','/etc/exports.dpkg-dist')
                shutil.copy('/usr/share/mythbuntu/examples/exports.dist',
                            '/etc/exports')

    def user_scripted_changes(self,reconfigure):
        """Local changes that can be performed by the user account.
           This function will be ran by the frontend"""
        for item in reconfigure:
            if item == "x11vnc_password":
                self.vnc.create_password(reconfigure[item])
